@@ -1,4 +1,4 @@
-# $Id: Parser.pm,v 1.19 2000/04/17 17:08:52 matt Exp $
+# $Id: Parser.pm,v 1.23 2000/05/16 16:53:37 matt Exp $
 
 package XML::XPath::Parser;
 
@@ -49,6 +49,7 @@ sub new {
 	$self->{context_size} = 0; # total size of context
 	$self->{vars} = {};
 	$self->{direction} = 'forward';
+	$self->{cache} = {};
 	return $self;
 }
 
@@ -98,12 +99,17 @@ sub my_sub {
 sub parse {
 	my $self = shift;
 	my $path = shift;
+	if ($self->{cache}{$path}) {
+		return $self->{cache}{$path};
+	}
 	my $tokens = $self->tokenize($path);
 
 	$self->{_tokpos} = 0;
 	my $tree = $self->analyze($tokens);
 	
-	debug("PARSED Expr to:\n", $tree->as_string, "\n");
+	$self->{cache}{$path} = $tree;
+	
+	debug("PARSED Expr to:\n", $tree->as_string, "\n") if $XML::XPath::Debug;
 	
 	return $tree;
 }
@@ -128,10 +134,10 @@ sub tokenize {
 			\.| # match current
 			($AXIS_NAME)?$NODE_TYPE| # match tests
 			processing-instruction|
-			\@$QName| # match attrib
+			\@($QName|\*)| # match attrib
 			\$$QName| # match variable reference
 			($AXIS_NAME)?(\*|$NCName\:\*|$QName)| # match NCName,NodeType,Axis::Test
-			<=|\-|>=|\/\/|and|or|mod|div| # multi-char seps
+			\!=|<=|\-|>=|\/\/|and|or|mod|div| # multi-char seps
 			[,\+=\|<>\/\(\[\]\)]| # single char seps
 			(?<!(\@|\(|\[))\*| # multiply operator rules (see xpath spec)
 			(?<!::)\*|
@@ -610,7 +616,7 @@ sub Step {
 					XML::XPath::Literal->new($1));
 			match($self, $tokens, '\\)', 1);
 		}
-		elsif ($token =~ /^\@($QName)$/o) {
+		elsif ($token =~ /^\@($QName|\*)$/o) {
 			$self->{_tokpos}++;
 			$step = XML::XPath::Step->new($self, 'attribute', $1);
 		}
@@ -653,7 +659,7 @@ sub is_step {
 	if ($token eq 'processing-instruction') {
 		return 1;
 	}
-	elsif ($token =~ /^\@($QName)$/o) {
+	elsif ($token =~ /^\@($QName|\*)$/o) {
 		return 1;
 	}
 	elsif ($token =~ /^$QName$/o && $tokens->[$self->{_tokpos}+1] ne '(') {
